==> ptypes-1.7.1.tbz: share/examples/ptypes/wshare/sysutils.cxx <==
/*
 *
 *  C++ Portable Types Library (PTypes)
 *  Version 1.7.1  Released 18-Oct-2002
 *
 *  Copyright (c) 2001, 2002 Hovik Melikyan
 *
 *  http://www.melikyan.com/ptypes/
 *  http://ptypes.sourceforge.net/
 *
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>

#ifdef WIN32
#  include <windows.h>
#  include <io.h>
#  include <direct.h>
#else
#  include <fcntl.h>
#  include <unistd.h>
#  include <sys/utsname.h>
#  include <dirent.h>
#  include <limits.h>
#  include <sys/param.h>
#  include <pwd.h>
#endif

#include <pport.h>

#include "sysutils.h"
#include "utils.h"


USING_PTYPES


static char* nodename = 0;
static char* username = 0;


char* get_nodename()
{
    if (nodename == 0)
    {
#ifdef WIN32
        char buf[MAX_COMPUTERNAME_LENGTH + 1];
        unsigned long bufsize = sizeof(buf);
        if (!GetComputerName(buf, &bufsize))
            fatal(1001, "GetComputerName() failed");
        nodename = strdup(buf);
#else
	struct utsname u;
	if (uname(&u) < 0)
	    fatal(1001, "Couldn't get the node name");
	nodename = strdup(u.nodename);
#endif
    }
    return nodename;
}


char* get_username()
{
    if (username == 0)
    {
#ifdef WIN32
        char buf[256 + 1];
        unsigned long bufsize = sizeof(buf);
        if (!GetUserName(buf, &bufsize))
            fatal(1002, "GetUserName() failed");
        username = strdup(buf);
#else
        struct passwd* pw = getpwuid(getuid());
        if (pw == 0)
	    fatal(1001, "Couldn't get the user name");
        username = strdup(pw->pw_name);
#endif
    }
    return username;
}


file_type_t get_file_type(const char* name)
{
    struct stat st;
    if (stat(name, &st) != 0)
        return FT_ERROR;
    if ((st.st_mode & S_IFDIR) == S_IFDIR)
        return FT_DIRECTORY;
    if ((st.st_mode & S_IFREG) == S_IFREG)
        return FT_FILE;
    return FT_OTHER;
}


#ifdef WIN32
bool is_symlink(const char*)
{
    return false;
#else
bool is_symlink(const char* name)
{
    struct stat st;
    if (lstat(name, &st) != 0)
        return false;
    return (st.st_mode & S_IFLNK) == S_IFLNK;
#endif
}



#ifdef WIN32
bool is_executable(const char* name)
{
    return stricmp(get_file_ext(name), ".exe") == 0;
#else
bool is_executable(const char* name)
{
    struct stat st;
    if (stat(name, &st) != 0)
        return false;
    return (st.st_mode & S_IXOTH) == S_IXOTH;
#endif
}



large get_file_size(const char* name)
{
    struct stat st;
    if (stat(name, &st) != 0)
        return -1;
    return st.st_size;
}


datetime get_file_mtime(const char* name)
{
    struct stat st;
    if (stat(name, &st) != 0)
        return invdatetime;
    return utodatetime(st.st_mtime);
}


#if defined(__sun__)


int daemon(int nochdir, int noclose) {
    int fd;
    
    switch (fork()) 
    {
    case -1:
        return -1;
    case 0:
        break;
    default:
        exit(0);
    }
    
    if (setsid() == -1)
        return -1;
    
    if (!nochdir)
        chdir("/");
    
    if (!noclose && (fd = open("/dev/null", O_RDWR, 0)) != -1) 
    {
        dup2(fd, STDIN_FILENO);
        dup2(fd, STDOUT_FILENO);
        dup2(fd, STDERR_FILENO);
        if (fd > 2)
            close(fd);
    }
    return 0;
}


#elif defined WIN32


int daemon(int nochdir, int)
{
    if (!nochdir)
        chdir("/");
    return 0;
}


#endif


file_info::file_info(const char* iname, large isize, datetime imodified)
    : name(iname), size(isize), modified(imodified)  {}


file_info::~file_info()
{
}


void get_directory(strlist& s, string path, bool dirs, int maxfiles)
{
#ifdef WIN32
    path += "*.*";
    _finddatai64_t f;
    int h = _findfirsti64((char*)pconst(path), &f); // this stupid typecast is for BCC
    if (h < 0)
        return;
    do
    {
        if (((f.attrib & _A_SUBDIR) != 0) == dirs)
        {
            string t = f.name;
            if (t == '.')
                continue;
            if (dirs)
                t += '/';
            add(s, t, new file_info(t, f.size, utodatetime(f.time_write)));
        }
    }
    while (_findnexti64(h, &f) == 0 && length(s) < maxfiles);
    _findclose(h);

#else
    if (path != '/' && trail_char(path) == '/')
	trunc_trail_char(path);
    DIR* dir = opendir(path);
    if (dir == 0)
	return;
    dirent* de;
    while ((de = readdir(dir)) != 0)
    {
	string name = de->d_name;
	if (name == '.')
	    continue;
	string fullname = path + '/' + name;

#ifdef __sun__
        file_type_t ft = get_file_type(fullname);
        if ((ft == FT_DIRECTORY) != dirs)
            continue;
        if (ft != FT_DIRECTORY && ft != FT_FILE && !is_symlink(fullname))
            continue;
#else
	if ((de->d_type == DT_DIR) != dirs)
	    continue;
	if (de->d_type != DT_DIR && de->d_type != DT_REG && de->d_type != DT_LNK)
	    continue;
#endif

	large size = 0;
	if (dirs)
	    name += '/';
	else
	    size = get_file_size(fullname);
	datetime mtime = get_file_mtime(fullname);
	add(s, name, new file_info(name, size, mtime));
    }
    closedir(dir);
#endif
}


string absolute_path(const string& rel)
{
    string abs;

#ifdef WIN32
    setlength(abs, 4096);
    char* p = unique(abs);
    _fullpath(p, rel, 4096);
    while (*p != 0)
    {
        if (*p == '\\')
            *p = '/';
        p++;
    }
#else
#  ifdef PATH_MAX
    int path_max = PATH_MAX;
#  else
    int path_max = pathconf(path, _PC_PATH_MAX);
	if (path_max <= 0)
            path_max = 4096;
#  endif
    setlength(abs, path_max);
    char* p = unique(abs);
    realpath(rel, p);
#endif

    return pconst(abs);
}


string get_file_ext(const string& name)
{
    const char* e = pconst(name) + length(name);
    int len = 0;
    while (e > pconst(name) && *e != '.' && *e != '/')
    {
        e--;
        len++;
    }
    if (*e == '.' && len > 1)
        return string(e, len);
    else
        return emptystr;
}

==> ptypes-1.7.5.tbz: share/examples/ptypes/wshare/sysutils.cxx <==
/*
 *
 *  C++ Portable Types Library (PTypes)
 *  Version 1.7.5   Released 9-Mar-2003
 *
 *  Copyright (c) 2001, 2002, 2003 Hovik Melikyan
 *
 *  http://www.melikyan.com/ptypes/
 *  http://ptypes.sourceforge.net/
 *
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>

#ifdef WIN32
#  include <windows.h>
#  include <io.h>
#  include <direct.h>
#else
#  include <fcntl.h>
#  include <unistd.h>
#  include <sys/utsname.h>
#  include <dirent.h>
#  include <limits.h>
#  include <sys/param.h>
#  include <pwd.h>
#endif

#include <pport.h>

#include "sysutils.h"
#include "utils.h"


USING_PTYPES


static char* nodename = 0;
static char* username = 0;


char* get_nodename()
{
    if (nodename == 0)
    {
#ifdef WIN32
        char buf[MAX_COMPUTERNAME_LENGTH + 1];
        unsigned long bufsize = sizeof(buf);
        if (!GetComputerName(buf, &bufsize))
            fatal(1001, "GetComputerName() failed");
        nodename = strdup(buf);
#else
	struct utsname u;
	if (uname(&u) < 0)
	    fatal(1001, "Couldn't get the node name");
	nodename = strdup(u.nodename);
#endif
    }
    return nodename;
}


char* get_username()
{
    if (username == 0)
    {
#ifdef WIN32
        char buf[256 + 1];
        unsigned long bufsize = sizeof(buf);
        if (!GetUserName(buf, &bufsize))
            fatal(1002, "GetUserName() failed");
        username = strdup(buf);
#else
        struct passwd* pw = getpwuid(getuid());
        if (pw == 0)
	    fatal(1001, "Couldn't get the user name");
        username = strdup(pw->pw_name);
#endif
    }
    return username;
}


file_type_t get_file_type(const char* name)
{
    struct stat st;
    if (stat(name, &st) != 0)
        return FT_ERROR;
    if ((st.st_mode & S_IFDIR) == S_IFDIR)
        return FT_DIRECTORY;
    if ((st.st_mode & S_IFREG) == S_IFREG)
        return FT_FILE;
    return FT_OTHER;
}


#ifdef WIN32
bool is_symlink(const char*)
{
    return false;
#else
bool is_symlink(const char* name)
{
    struct stat st;
    if (lstat(name, &st) != 0)
        return false;
    return (st.st_mode & S_IFLNK) == S_IFLNK;
#endif
}



#ifdef WIN32
bool is_executable(const char* name)
{
    return stricmp(get_file_ext(name), ".exe") == 0;
#else
bool is_executable(const char* name)
{
    struct stat st;
    if (stat(name, &st) != 0)
        return false;
    return (st.st_mode & S_IXOTH) == S_IXOTH;
#endif
}



large get_file_size(const char* name)
{
    struct stat st;
    if (stat(name, &st) != 0)
        return -1;
    return st.st_size;
}


datetime get_file_mtime(const char* name)
{
    struct stat st;
    if (stat(name, &st) != 0)
        return invdatetime;
    return utodatetime(st.st_mtime);
}


#if defined(__sun__)


int daemon(int nochdir, int noclose) {
    int fd;
    
    switch (fork()) 
    {
    case -1:
        return -1;
    case 0:
        break;
    default:
        exit(0);
    }
    
    if (setsid() == -1)
        return -1;
    
    if (!nochdir)
        chdir("/");
    
    if (!noclose && (fd = open("/dev/null", O_RDWR, 0)) != -1) 
    {
        dup2(fd, STDIN_FILENO);
        dup2(fd, STDOUT_FILENO);
        dup2(fd, STDERR_FILENO);
        if (fd > 2)
            close(fd);
    }
    return 0;
}


#elif defined WIN32


int daemon(int nochdir, int)
{
    if (!nochdir)
        chdir("/");
    return 0;
}


#endif


file_info::file_info(const char* iname, large isize, datetime imodified)
    : name(iname), size(isize), modified(imodified)  {}


file_info::~file_info()
{
}


void get_directory(filist& s, string path, bool dirs, int maxfiles)
{
#ifdef WIN32
    path += "*.*";
    _finddatai64_t f;
    int h = _findfirsti64((char*)pconst(path), &f); // this stupid typecast is for BCC
    if (h < 0)
        return;
    do
    {
        if (((f.attrib & _A_SUBDIR) != 0) == dirs)
        {
            string t = f.name;
            if (t == '.')
                continue;
            if (dirs)
                t += '/';
            add(s, t, new file_info(t, f.size, utodatetime(f.time_write)));
        }
    }
    while (_findnexti64(h, &f) == 0 && length(s) < maxfiles);
    _findclose(h);

#else
    if (path != '/' && trail_char(path) == '/')
	trunc_trail_char(path);
    DIR* dir = opendir(path);
    if (dir == 0)
	return;
    dirent* de;
    while ((de = readdir(dir)) != 0)
    {
	string name = de->d_name;
	if (name == '.')
	    continue;
	string fullname = path + '/' + name;

#ifdef __sun__
        file_type_t ft = get_file_type(fullname);
        if ((ft == FT_DIRECTORY) != dirs)
            continue;
        if (ft != FT_DIRECTORY && ft != FT_FILE && !is_symlink(fullname))
            continue;
#else
	if ((de->d_type == DT_DIR) != dirs)
	    continue;
	if (de->d_type != DT_DIR && de->d_type != DT_REG && de->d_type != DT_LNK)
	    continue;
#endif

	large size = 0;
	if (dirs)
	    name += '/';
	else
	    size = get_file_size(fullname);
	datetime mtime = get_file_mtime(fullname);
	add(s, name, new file_info(name, size, mtime));
    }
    closedir(dir);
#endif
}


string absolute_path(const string& rel)
{
    string abs;

#ifdef WIN32
    setlength(abs, 4096);
    char* p = unique(abs);
    _fullpath(p, rel, 4096);
    while (*p != 0)
    {
        if (*p == '\\')
            *p = '/';
        p++;
    }
#else
#  ifdef PATH_MAX
    int path_max = PATH_MAX;
#  else
    int path_max = pathconf(path, _PC_PATH_MAX);
	if (path_max <= 0)
            path_max = 4096;
#  endif
    setlength(abs, path_max);
    char* p = unique(abs);
    realpath(rel, p);
#endif

    return pconst(abs);
}


string get_file_ext(const string& name)
{
    const char* e = pconst(name) + length(name);
    int len = 0;
    while (e > pconst(name) && *e != '.' && *e != '/')
    {
        e--;
        len++;
    }
    if (*e == '.' && len > 1)
        return string(e, len);
    else
        return emptystr;
}
